from flask import Flask, render_template, jsonify, request, Response
import queue
import time
import threading
import json

app = Flask(__name__)

# Initial state
status = {"state": "Here", "color": "green", "message": "Here"}
clients = []

def notify_clients():
    for client in clients:
        client.put(status)

@app.route('/')
def home():
    return render_template('index.html', status=status)

@app.route('/toggle', methods=['POST'])
def toggle_status():
    global status
    if status["state"] == "Here":
        status = {"state": "Away", "color": "red", "message": "Away"}
    else:
        status = {"state": "Here", "color": "green", "message": "Here"}
    notify_clients()
    return jsonify(status)

@app.route('/set_message', methods=['POST'])
def set_message():
    global status
    custom_message = request.json.get('message')
    if custom_message:
        status["message"] = custom_message
    notify_clients()
    return jsonify(status)

@app.route('/stream')
def stream():
    def event_stream():
        client_queue = queue.Queue()
        clients.append(client_queue)
        try:
            while True:
                data = client_queue.get()
                yield f"data: {json.dumps(data)}\n\n"
        except GeneratorExit:
            clients.remove(client_queue)

    return Response(event_stream(), content_type='text/event-stream')

if __name__ == '__main__':
    app.run(debug=True, threaded=True)
